#!/usr/bin/env python

from compiler import ast

class ASTVisitor:

    """
    Performs a depth-first walk of the AST.

    The ASTVisitor is responsible for walking over the tree in the correct
    order. For each node, it calls the 'visit' method on the node, and this
    method is then responsible to calling an appropriate method on the visitor.

    For example, where 'Class' is the name of the node's class, the node's
    'visit' method might invoke a 'visitClass' method on the visitor, although
    it need not follow this particular naming convention.
    """

    def __init__(self):
        self.node = None
        self.visitor = self

    def default(self, node, *args):
        for child in node.getChildNodes():
            self.dispatch(child, *args)

    def dispatch(self, node, *args):
        self.node = node
        try:
            return node.visit(self.visitor, *args)
        except AttributeError:
            return self.visitor.default(node, *args)

    def preorder(self, tree, visitor, *args):

        "Do preorder walk of tree using visitor."

        self.visitor = visitor
        visitor.visit = self.dispatch

        # NOTE: *args not exposed by the walk function.

        self.dispatch(tree, *args)

_walker = ASTVisitor

def walk(tree, visitor, walker=None):
    if walker is None:
        walker = _walker()
    walker.preorder(tree, visitor)
    return walker.visitor

# vim: tabstop=4 expandtab shiftwidth=4
